import connexion
from flask_cors import CORS

# from .fakemock_resolver import FakerMockResolver
from .apiman_resolver import ApimanMockResolver


class Mock(object):

    def __init__(self, spec, spec_dir='openapi/'):
        self.app = connexion.FlaskApp(__name__, specification_dir=spec_dir)
        CORS(self.app.app)
        self.app.app.url_map.strict_slashes = False
        self.spec = spec

    def start(self, resolver=ApimanMockResolver, port=8080, **kwargs):
        self.app.add_api(self.spec, resolver=resolver('api'))
        self.app.run(port=port, **kwargs)



from .requestitem import RequestItem
from ..errors import FolderParserException, RequestParserException
from ..utils import is_folder


class Folder(object):

    def __init__(self, folder=None):
        if (folder is None) \
                or (not isinstance(folder, dict) \
                    or (not is_folder(folder))):
            raise FolderParserException('Not a valid folder')
        self.folder = folder

    @classmethod
    def parse(cls, folder=None):
        return cls(folder)

    @property
    def summary(self):
        return self.get_summary()

    @property
    def description(self):
        return self.get_description()

    def get_summary(self):
        return self.folder.get('name')

    def get_description(self):
        return self.folder.get('description')

    def get_requestitems(self, folder=None):
        request_items = []
        folder = folder or self.folder['item']
        for item in folder:
            if is_folder(item):
                request_items.extend(self.get_requestitems(item.get('item')))
            try:
                request_items.append(RequestItem(item))
            except RequestParserException:
                continue
        return request_items



from .request import Request
from .response import Response
from ..errors import RequestItemParserException
from ..utils import is_requestitem


class RequestItem(object):
    def __init__(self, requestitem=None):
        if (requestitem is None) \
                or (not isinstance(requestitem, dict) \
                    or (not is_requestitem(requestitem))):
            raise RequestItemParserException('Not a valid request')
        self.requestitem = requestitem
        self.request = Request(requestitem['request'])
        self.responses = [Response(item) for item in requestitem['response']]

    @classmethod
    def parse(cls, requestitem=None):
        return cls(requestitem)

    @property
    def summary(self):
        return self.get_summary()

    @property
    def description(self):
        return self.get_description()

    def get_summary(self):
        return self.requestitem.get('name')

    def get_description(self):
        return self.request.description

    def get_request(self):
        return self.request

    def get_responses(self):
        return self.responses



import json

import jsonpath_rw

from .utils import python2schematypes


class SchemaConvertor(object):

    def filter_unique_schema(self, schemas):
        uniq = []
        prevschema = None
        while len(schemas):
            newschema = schemas.pop(0)
            if prevschema == newschema:
                continue
            prevschema = newschema
            uniq.append(newschema)
        return uniq

    @property
    def IGNOREPROP(self):
        return f'__{self.__class__.__name__}__'

    @property
    def IGNOREPROPKEYVAL(self):
        return f'__{self.IGNOREPROP}__all__'

    def _walker(self, item, ignore=False):
        if item is None:
            return {}
        (_type, _format) = python2schematypes[type(item)]
        schema = dict(type=_type)
        if isinstance(item, str):
            if item == self.IGNOREPROP:
                schema = dict(additionalProperties={})
        elif isinstance(item, int):
            schema['format'] = _format
        elif isinstance(item, dict):
            schema['properties'] = dict()
            if (len(item.keys()) > 0):
                schema['required'] = []
            for k, v in item.items():
                if v == self.IGNOREPROPKEYVAL:
                    schema['properties']['additionalProperties'] = {}
                else:
                    schema['required'].append(k)
                    schema['properties'][k] = self._walker(v)
        elif isinstance(item, list):
            uniqueschemas = self.filter_unique_schema(
                [self._walker(v) for v in item])
            schema['items'] = dict()
            if len(uniqueschemas) > 1:
                schema['items']['allOf'] = uniqueschemas
            elif uniqueschemas:
                schema['items'] = uniqueschemas[0]
        return schema

    def parse_ignore(self, expr):
        type_explode = expr.split(':')
        if len(type_explode) > 1 and type_explode[-1] == 'a':
            return ''.join(type_explode[:-1]), True
        return expr, False

    def filter_item(self, item, ignores=[]):
        for ignore in ignores:
            ignore_expr, ignore_all = self.parse_ignore(ignore)
            jsonpath_expr = jsonpath_rw.parse(ignore_expr)
            matches = jsonpath_expr.find(item)
            ignoreprop = self.IGNOREPROPKEYVAL if ignore_all else self.IGNOREPROP
            for match in matches:
                item = self.json_update_path(item, self.json_get_path(match), ignoreprop)
        return item

    def json_get_path(self, match):
        '''return an iterator based upon MATCH.PATH. Each item is a path component,
        start from outer most item.'''
        if match.context is not None:
            for path_element in self.json_get_path(match.context):
                yield path_element
            yield str(match.path)

    def json_update_path(self, json, path, value):
        '''Update JSON dictionnary PATH with VALUE. Return updated JSON'''
        try:
            first = next(path)
            # check if item is an array
            if first.startswith('[') and first.endswith(']'):
                try:
                    first = int(first[1:-1])
                except ValueError:
                    pass
            json[first] = self.json_update_path(json[first], path, value)
            return json
        except StopIteration:
            return value

    @classmethod
    def convert(cls, item, format_to=None, ignore=[]):
        # Mark the item for filtering
        item = cls().filter_item(item, ignore)
        schema = cls()._walker(item)
        if format_to and format_to in ['json']:
            return json.dumps(schema)
        return schema



import json
import os

import click
import yaml

from . import from_collection
from .mock import Mock
from .parser import CollectionParser
from .spec import Spec


@click.group(help="Convert or mock your postman collection to openapi schema")
@click.pass_context
def cli(ctx):
    pass


@cli.command()
@click.option("--format", '-f', default='yaml', help="Format to output. One of json or yaml. Defaults to yaml")
@click.option("--ignore", '-i', help="Ignore file in yaml or json")
@click.argument('POSTMANFILE', required=True)
@click.argument('OUTFILE', required=True)
def convert(format, ignore, postmanfile, outfile):
    collection_file = os.path.join(postmanfile)
    try:
        ignore_file = os.path.join(ignore)
    except TypeError:
        ignore_file = None

    collection = CollectionParser(from_collection(collection_file))
    spec = Spec(collection)
    converted, server = spec.convert(ignorespec=ignore_file, yaml=(format == 'yaml'))
    converted = yaml.load(converted)
    converted.update(server)
    if outfile:
        with open(outfile, 'w') as f:
            yaml.dump(converted, f, default_flow_style=False)
            # f.write(converted)

    click.echo(click.style('Schema converted successfully!', fg='green'))
    return json.dumps(converted) if format == 'json' else converted


@cli.command()
@click.option("--host", '-h', default='127.0.0.1', help="Host Default: 127.0.0.1")
@click.option("--port", '-p', default=8080, help="Port Default: 8080")
@click.option("--debug", '-D', default=False, help="Debug", is_flag=True)
@click.argument('SPECFILE', required=True)
def mock(host, port, debug, specfile):
    mock = Mock(specfile, spec_dir=os.path.abspath('.'))
    mock.start(port=port, host=host, debug=debug)


if __name__ == '__main__':
    cli()



from apispec import APISpec

from .operation import Operation
from .plugins import MultiOperationBuilderPlugin
import re


class Spec(object):
    OPENAPI_VERSION = '3.0.0'

    def __init__(self, postman_collection):
        self.postman_collection = postman_collection
        self._servers = ['http://localhost']

    @property
    def info(self):
        return self.get_info()

    @property
    def openapi(self):
        return Spec.OPENAPI_VERSION

    @property
    def servers(self):
        return self.get_servers()

    def add_servers(self, server):
        if isinstance(server, list):
            self._servers.extend(server)
        else:
            self._servers.append(server)
        return self

    def get_info(self, version='1.0.0'):
        return dict(
            title=self.postman_collection.name,
            description=self.postman_collection.description,
            version=version
        )

    def get_servers(self):
        return self._servers

    def convert(self, yaml=False, references=True, ignorespec=None, **options):
        spec = APISpec(
            title=self.info.get('title'),
            version=self.info.get('version'),
            openapi_version=self.openapi,
            plugins=[MultiOperationBuilderPlugin(references)],
            **options
        )
        results = []  # use a normal dictionary for our output
        server_url = []
        for requestitem in self.postman_collection.get_requestitems():
            try:
                requestitem_dict = requestitem.__dict__
                raw_url = requestitem_dict['requestitem']['request']['url']['raw']
                raw_path = '/'.join(requestitem_dict['requestitem']['request']['url']['path'])
                s_url = re.sub(raw_path,'',raw_url)
                if s_url[-1] == "/":
                    server_url.append(s_url[:-1])
                else:
                    server_url.append(s_url)
                spec.path(
                    path=requestitem.get_request().path,
                    operations=Operation(requestitem, ignorespec=ignorespec).get()
                )


            except Exception:
                pass
        for server in set(server_url):
            results.append({"url": server})
        server_list = {"servers": results}
        print(server_list)
        return spec.to_yaml() if yaml else spec.to_dict(), server_list



from ..errors import RequestParserException
from ..utils import filter_env_var, is_request, format_path, \
    map_to_dict

# Postman to Content-Type mapping
request_contenttype_map = {
    "raw": '*/*',
    "text": "text/plain",
    "json": "application/json",
    "urlencoded": 'application/x-www-form-urlencoded',
    "formdata": 'multipart/form-data',
}


class Request(object):

    def __init__(self, request=None):
        if (request is None) \
                or (not isinstance(request, dict) \
                    or (not is_request(request))):
            raise RequestParserException('Not a valid request')
        self.request = request

    @classmethod
    def parse(cls, request=None):
        return cls(request)

    @property
    def description(self):
        return self.get_description()

    @property
    def path(self):
        return self.get_path(True)

    @property
    def method(self):
        return self.get_method()

    @property
    def query_string(self):
        return self.get_query_string()

    @property
    def headers(self):
        return self.get_headers()

    @property
    def body(self):
        return self.get_body()

    @property
    def body_contenttype(self):
        return self.get_body_contenttype()

    def get_description(self):
        return self.request.get('description')

    def get_path(self, formatted=False):
        try:
            path = list(map(
                lambda path: path[1:-1] if filter_env_var(path) else path,
                self.request['url']['path']))
        except:
            path = []
        return format_path(path) if formatted else path

        # Return LOWERCASED method name

    # @TODO Make some constant in class to refer method name
    # so that other classes can use constants instead of hardcoded
    def get_method(self):
        return self.request['method'].lower() if 'method' in self.request else None

    def get_query_string(self):
        if self.request['url'].get('query', None) is None:
            return None
        return map_to_dict(self.request['url'].get('query'))

    def get_headers(self, header=None):
        if header is None:
            return map_to_dict(self.request['header'])
        for name, value in map_to_dict(self.request['header']).items():
            if name.lower() == header.lower():
                return value
        return None

    def get_body(self):
        if 'body' in self.request:
            bodyitem = self.request['body'][self.request['body']['mode']]
            if isinstance(bodyitem, list):
                return map_to_dict(bodyitem)
            return bodyitem
        return None

    def get_body_contenttype(self, default='*/*'):
        mode = self.request['body']['mode']
        # We try to deduce from given langugae
        if mode == 'raw' and 'options' in self.request['body']:
            if 'language' in self.request['body']['options']:
                mode = self.request['body']['options']['language'].get(mode, 'raw')
        return request_contenttype_map.get(mode, default)



class OpenmanException(Exception):
    pass


class RequestParserException(OpenmanException):
    pass


class ResponseParserException(OpenmanException):
    pass


class CollectionParserException(OpenmanException):
    pass


class FolderParserException(OpenmanException):
    pass


class RequestItemParserException(OpenmanException):
    pass



import json
import os

import yaml

from .errors import OpenmanException
from .schema_converter import SchemaConvertor


def from_collection(postmanfile=None):
    if postmanfile is None:
        raise OpenmanException('Empty file is not allowed')
    if not os.path.isfile(postmanfile):
        raise FileNotFoundError("Postman collection not found")
    with open(postmanfile, 'r') as f:
        return json.load(f)


def from_ignore(ignorefile=None):
    ignoreschema = {}
    if not ignorefile:
        return ignoreschema
    with open(ignorefile, 'r') as f:
        try:
            ignoreschema = yaml.safe_load(f)
        except yaml.YAMLError as exc:
            ignoreschema = json.load(f)
    return ignoreschema



import random
from connexion.apis.flask_api import FlaskApi
from connexion.http_facts import FORM_CONTENT_TYPES
from connexion.mock import MockResolver
from connexion.utils import all_json
from faker import Faker

from ..utils import strip_charset, sanitized


class FakerMockResolver(MockResolver):
    def __init__(self, mock_all):
        super().__init__(mock_all)
        self.faker = Faker()

    def example_resolver(self, operation):
        linked_responses = []

        consumes = operation.consumes
        request = FlaskApi.get_request()

        if all_json(consumes):
            request_body = request.json
        elif consumes[0] in FORM_CONTENT_TYPES:
            request_body = {sanitized(k): v for k, v in request.form.items()}
        else:
            request_body = request.body

        try:
            query = request.query.to_dict(flat=False)
        except AttributeError:
            query = dict(request.query.items())

        content_type_header = request.headers.get('Content-Type')
        content_type_header = content_type_header or request.headers.get('Accept', '*/*')
        content_type_header = strip_charset(content_type_header)

        # first priority - Request body
        if request_body:
            try:
                operation_request_body = operation.request_body['content'][content_type_header]
                for item in operation_request_body.get('x-link-response'):
                    if dict(item.get('value')) == request_body:
                        linked_responses.append(item.get('x-response-id'))
            except KeyError:
                pass

        # second priority - Query params
        if query:
            query_params = filter(lambda x: x.get('in') == 'query', operation.parameters)
            for param in query_params:
                param_val = query.get(param.get('name'), None)
                if param_val is not None:
                    for item in param.get('x-link-response'):
                        if item.get('value') in param_val:
                            linked_responses.append(item.get('x-response-id'))

        # third priority - Headers
        if content_type_header:
            header_params = filter(lambda x: x.get('in') == 'header', operation.parameters)
            for param in header_params:
                if param.get('name').lower() in ['accept', 'content-type']:
                    for item in param.get('x-link-response'):
                        if item.get('value') == content_type_header:
                            linked_responses.append(item.get('x-response-id'))

        return linked_responses

    def _fake(self, schema, example=None):
        # use faker to generate fake data from jsonschema
        example = example or schema.get('example')

        if 'oneOf' in schema:
            return self._fake(schema['oneOf'][random.randint(0, (len(schema['oneOf']) - 1))], example)

        if ('additionalProperties' in schema) or (schema == {}):
            return example or getattr(self.faker, 'pydict')()

        if schema["type"].lower() == "array":
            if isinstance(example, list) and len(example):
                return [self._fake(schema["items"], example[i]) for i in range(len(example))]
            else:
                return [self._fake(schema["items"]) for i in range(2)]

        if schema["type"].lower() == "object":
            if example:
                return {k: self._fake(v, example.get(k)) for k, v in schema["properties"].items()}
            else:
                return {k: self._fake(v) for k, v in schema["properties"].items()}

        if "enum" in schema:
            # if enum, just choose one of enum items at random
            return random.choice(schema["enum"])

        if schema["type"].lower() == "string":
            if example:
                return str(example)
            return getattr(self.faker, schema.get("format", "word"))()

        if schema["type"].lower().startswith("int"):
            if example:
                return int(example)
            return int(example) or self.faker.random_int()

        return getattr(self.faker, schema.get("type", "word"))()

    def example_filter(self, schema, response_id):
        if 'examples' in schema:
            for name, example in schema['examples'].items():
                if example.get('x-response-id') == response_id:
                    return example.get('value')
        elif 'example' in schema:
            if schema['example'].get('x-response-id') == response_id:
                return schema['example'].get('value')
        return None

    def mock_operation(self, operation, *args, **kwargs):
        _schema = _example = None
        linked_responses = self.example_resolver(operation)

        if len(linked_responses):
            linked_response = linked_responses[random.randint(0, (len(linked_responses) - 1))]
            for code, response in operation.responses.items():
                for media_type, schema in response['content'].items():
                    _example = self.example_filter(schema, linked_response)
                    status_code = code

        if not _example:
            _example, status_code = operation.example_response()

        if not _example:
            status_code = sorted(operation._responses.keys())[0]
            _schema = operation.response_schema(status_code, operation.get_mimetype())

        status_code = int(status_code) or 200

        if _example:
            return _example, status_code

        # If there are no example,  we try to fake from schema
        return self._fake(_schema or {}), status_code



import re

from .folder import Folder
from .requestitem import RequestItem
from ..errors import CollectionParserException
from ..utils import is_collection, is_folder, is_requestitem


class Collection(object):

    def __init__(self, collection=None):
        if (collection is None) \
                or (not isinstance(collection, dict) \
                    or (not is_collection(collection))):
            raise CollectionParserException('Not a valid collection')
        self.collection = collection

    @classmethod
    def parse(cls, collection=None):
        return cls(collection)

    @property
    def version(self):
        return self.get_version()

    @property
    def name(self):
        return self.get_name()

    @property
    def description(self):
        return self.get_description()

    def get_version(self):
        search = re.search(
            '^http[s].\\/\\/.*\\/(.*)\\/collection\\.json',
            self.collection['info']['schema'])
        if search and search.group(1):
            return search.group(1)
        return None

    def get_name(self, default='TODO: API Specification'):
        return self.collection['info'].get('name', default)

    def get_description(self, default='TODO: API Specification description'):
        return self.collection['info'].get('description', default)

    def get_requestitems(self):
        items = []
        for item in self.collection['item']:
            if is_requestitem(item):
                items.append(RequestItem(item))
            elif is_folder(item):
                folder = Folder(item)
                items.extend(folder.get_requestitems())
        return items

    def get_folders(self):
        folders = []
        for item in self.collection['item']:
            if is_folder(item):
                folders.append(Folder(item))
        return folders

    # def validateSchema(self):
    #     schemaVer = self.schemaVersion.replace('v', '')
    #     schemaPath = os.path.realpath(
    #         os.path.join(os.getcwd(), os.path.dirname(__file__), '..'))
    #     with open(f'{schemaPath}/schema/{schemaVer}.json') as f:
    #         try:
    #             schema = json.load(f)
    #         except Exception:
    #             return True
    #     if schema:
    #         jsonschema.validate(instance=self.pmcollection, schema=schema)



from connexion.apis.flask_api import FlaskApi
from connexion.mock import MockResolver

MIME_TYPES = {
    'json': 'application/json',
    'xml': 'application/xml',
    'text': 'text/plain',
    'html': 'text/html',
}


class ApimanMockResolver(MockResolver):
    def __init__(self, mock_all):
        super().__init__(mock_all)

    def mock_operation(self, operation, *args, **kwargs):
        request = FlaskApi.get_request()
        status_code = request.query.get('__code', None)
        content_type = request.query.get('__type', None)

        resp, code = operation.example_response(status_code, content_type)
        if resp is not None:
            return resp, int(code)
        return 'No example response was defined.', int(code)



from ..schema_converter import SchemaConvertor
from ..utils import camelize, DEFAULT_STATUS_CODE


class Operation(object):

    def __init__(self, request_item, ignorespec=None):
        self.request_item = request_item
        self.ignorespec = ignorespec

    def get_path(self, normalized=False):
        return self.request_item.get_request().get_path(normalized)

    def get_method(self):
        return self.request_item.get_request().get_method()

    def id(self):
        return camelize(self.get_path(True) + ' ' + self.get_method())

    def get_ignores(self, path, method, code):
        if not self.ignorespec:
            return []
        if 'schema' not in self.ignorespec:
            return []
        for _path, schemas in self.ignorespec['schema'].items():
            if camelize(_path) == camelize(str(path)):
                for _method, responsecode in schemas.items():
                    if _method == method:
                        return responsecode.get(int(code), [])
        return []

    def get(self):
        _operation = dict()
        method = self.get_method()
        _operation[method] = dict(
            operationId=self.id(),
            summary=self.request_item.summary,
        )

        if self.request_item.description:
            _operation[method]['description'] = self.request_item.description

        # params
        params = self.parameters()
        if len(params):
            _operation[method]['parameters'] = params

        # request body
        requestbody = self.request_body()
        if requestbody:
            _operation[method]['requestBody'] = requestbody

        # responses
        _operation[method]['responses'] = self.responses() or \
                                          {DEFAULT_STATUS_CODE: {'description': ''}}

        return _operation

    def responses(self):
        response_schema = dict()
        for response in self.request_item.get_responses():
            ignoreschema = self.get_ignores(self.get_path(), self.get_method(), response.code)
            schema = SchemaConvertor.convert(response.body, ignore=ignoreschema)
            exampleDescription = self.id() + str(response.code)
            response_content = dict()
            response_content[response.content_type] = dict(
                schema=schema,
                examples={
                    camelize(response.description): {
                        'value': response.body,
                        'x-response-id': response.id
                    },
                },
            )
            response_schema[int(response.code)] = dict(
                content=response_content,
                description=response.description
            )
        return response_schema

    def append_param_example(self, location, name, responses=None):
        """
        This extracts the param `name` in `location` (ex- query, header etc)
        from resonse example and append them to the location
        """
        if not responses:
            responses = self.request_item.get_responses()
        links = []
        for response in responses:
            example_request = response.get_example()

            try:
                for _name, _value in getattr(example_request, location).items():
                    if name == _name:
                        links.append({
                            'value': _value,
                            'x-response-id': response.id
                        })
                        break
            except AttributeError as e:
                pass
        return links

    def append_body_example(self, location, name, responses=None):
        """
        This extracts the param `name` in `location` (ex- query, header etc)
        from resonse example and append them to the location
        """
        if not responses:
            responses = self.request_item.get_responses()
        links = []
        for response in responses:
            example_request = response.get_example()
            if example_request.body:
                links.append({
                    'value': example_request.body,
                    'x-response-id': response.id
                })
        return links

    def parameters(self):
        """
        Constructs OpenAPI Spec compatible parameters spec
        Parameters can be one of four: path, cookie, query, header
        @link https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.2.md#parameter-locations
        """
        params = []
        request = self.request_item.get_request()
        responses = self.request_item.get_responses()
        # Build Query params
        if isinstance(request.query_string, dict):
            for name, value in request.query_string.items():
                query_params = dict()
                query_params['in'] = 'query'
                query_params['name'] = name
                query_params['schema'] = SchemaConvertor.convert(value)
                query_params['schema'].update(dict(example=value))
                query_params['x-link-response'] = []
                # Check if any response has this param in their request
                query_params['x-link-response'] = self.append_param_example('query_string', name, responses)

                params.append(query_params)

        # Build headers params
        if isinstance(request.headers, dict):
            for name, value in request.headers.items():
                header_params = dict()
                header_params['in'] = 'header'
                header_params['name'] = name
                header_params['schema'] = SchemaConvertor.convert(value)
                header_params['schema'].update(dict(example=value))
                header_params['x-link-response'] = self.append_param_example('headers', name, responses)
                params.append(header_params)

        # Build Path params
        # Build cookie params

        return params

    def request_body(self):
        request_body = dict()
        request = self.request_item.get_request()
        responses = self.request_item.get_responses()
        if request.body:
            bodyschema = SchemaConvertor.convert(request.body)
            header = request.get_headers('content-type') or '*/*'
            request_body['content'] = {
                header: {
                    'schema': bodyschema,
                    'example': dict(value=request.body),
                    'x-link-response': self.append_body_example('body', request.body, responses)
                }
            }

        return request_body



from .collection import Collection as CollectionParser
from .folder import Folder as FolderParser
from .request import Request as RequestParser
from .requestitem import RequestItem as RequestItemParser
from .response import Response as ResponseParser



from apispec import BasePlugin
from apispec.exceptions import DuplicateComponentNameError

from ..utils import camelize


class MultiOperationBuilderPlugin(BasePlugin):
    def __init__(self, refs=False):
        self.refs = refs
        self._reserved_keys = ['oneOf', 'allOf', 'anyOf']
        self.counter = {'schema': {}, 'example': {}}

    def init_spec(self, spec):
        super().init_spec(spec)
        self.spec = spec
        self.duplicate_schemas = {}
        self.duplicate_examples = {}

    def add_ref_schema(self, name, schema):
        # _name = self._get_uniq_refname('schema', name)
        try:
            self.spec.components.schema(name, schema)
        except DuplicateComponentNameError as e:
            name = self._get_uniq_refname('schema', name)
            self.add_ref_schema(name, schema)
        return self.spec.get_ref('schema', name)

    def add_ref_example(self, name, body):
        try:
            self.spec.components.example(name, body)
        except DuplicateComponentNameError as e:
            name = self._get_uniq_refname('example', name)
            self.add_ref_example(name, body)
        return self.spec.get_ref('example', name)

    def _get_uniq_refname(self, key, name):
        if self.counter[key].get(name, None) is None:
            self.counter[key] = {name: 0}
            _name = name
        else:
            self.counter[key][name] += 1
        _name = name + '_' + str(self.counter[key][name] + 1)
        return _name

    def _fix_response_schema(self, path, operations):
        for key, value in operations.items():
            for code, response in value.get('responses', {}).items():
                if 'content' in response:
                    for content_type, schema in response['content'].items():
                        opskey = camelize(path + ' ' + key + str(code)) + content_type

                        # schemafix
                        if self.duplicate_schemas.get(opskey, None) is None:
                            if self.refs:
                                schema['schema'] = self.add_ref_schema(
                                    'schema' + camelize(path + ' ' + key + str(code)), schema['schema'])
                            self.duplicate_schemas[opskey] = [schema['schema']]
                        else:
                            if self.refs:
                                schema['schema'] = self.add_ref_schema(
                                    'schema' + camelize(path + ' ' + key + str(code)), schema['schema'])
                            self.duplicate_schemas[opskey].append(schema['schema'])
                            schema['schema'] = dict(
                                oneOf=[
                                    schema for schema in self.duplicate_schemas.get(opskey)]
                            )
                        # examplefix
                        if 'examples' in schema:
                            if self.duplicate_examples.get(opskey, None) is None:
                                if self.refs:
                                    for desc, example_item in schema['examples'].items():
                                        schema['examples'][desc] = self.add_ref_example(
                                            'example' + camelize(path + ' ' + key + str(code)),
                                            example_item)
                                self.duplicate_examples[opskey] = [schema['examples']]
                            else:
                                if self.refs:
                                    for desc, example_item in schema['examples'].items():
                                        schema['examples'][desc] = self.add_ref_example(
                                            'example' + camelize(path + ' ' + key + str(code)),
                                            example_item)
                                self.duplicate_examples[opskey].append(schema['examples'])
                                for example in self.duplicate_examples.get(opskey):
                                    for example_name, example_value in example.items():
                                        schema['examples'][example_name] = example_value

    def operation_helper(self, path, operations, **kwargs):
        """Operation helper that add `deprecated` flag if in `kwargs`
        """
        self._fix_response_schema(path, operations)



import json
import uuid

from .request import Request
from ..errors import ResponseParserException, RequestParserException
from ..utils import is_response, map_to_dict, \
    strip_charset


class Response(object):

    def __init__(self, response=None):
        if (response is None) \
                or (not isinstance(response, dict) \
                    or (not is_response(response))):
            raise ResponseParserException('Not a valid response')
        self.response = response
        self.uuid = uuid.uuid4()

    @classmethod
    def parse(cls, response=None):
        return cls(response)

    @property
    def code(self):
        return self.get_code()

    @property
    def id(self):
        return self.get_id()

    @property
    def headers(self):
        return self.get_headers()

    @property
    def content_type(self):
        return self.get_content_type()

    @property
    def body(self):
        return self.get_body()

    @property
    def description(self):
        return self.get_description()

    @property
    def example(self):
        return self.get_example()

    def get_id(self):
        return str(self.uuid)

    def get_code(self, default_code=200):
        return self.response.get('code', default_code)

    def get_description(self):
        return self.response.get('name', '')

    def get_headers(self, header=None):
        """
        Return headers for given response.
        if `header` is provided, it returns
        the first matching header value
        """
        if header is None:
            return map_to_dict(self.response['header'])
        for name, value in map_to_dict(self.response.get('header') or []).items():
            # @TODO name.lower() in header.lower() might be better
            if header and (name.lower() == header.lower()):
                return value
        return None

    def get_content_type(self):
        header = self.get_headers('Content-Type') or '*/*'
        return strip_charset(header)

    def get_body(self):
        responsebody = self.response['body'] if 'body' in self.response \
            else None
        try:
            return json.loads(responsebody)
        except Exception:
            return responsebody

    def get_example(self):
        if 'originalRequest' in self.response:
            try:
                return Request(self.response['originalRequest'])
            except RequestParserException:
                pass
        return None



import re

_postman_env_regex = r"\{\{[a-zA-Z0-9_.]+\}\}"

DEFAULT_STATUS_CODE = 200

# https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.2.md#dataTypeFormat
python2schematypes = {
    int: ('integer', 'int32'),
    list: ('array', None),
    str: ('string', None),
    bool: ('boolean', "null"),
    dict: ('object', None),
}


def filter_env_var(item):
    return re.findall(_postman_env_regex, item)


def is_requestitem(item):
    return isinstance(item, dict) and \
           ('name' in item) and \
           ('request' in item) and \
           ('item' not in item)


def is_request(item):
    return isinstance(item, dict) and \
           ('method' in item) and \
           ('item' not in item)


def is_response(item):
    return isinstance(item, dict) and \
           ('originalRequest' in item) and \
           ('item' not in item)


def is_folder(item):
    return isinstance(item, dict) and \
           set(['name', 'item', 'description']).issubset(set(item.keys()))


def is_collection(item):
    return isinstance(item, dict) and \
           ('info' in item) and \
           ('schema' in item['info'])


def format_path(path):
    if len(path) > 0:
        path = '/'.join(path) if isinstance(path, list) \
            else path
        final_path = '/' + path if path[0] != '/' else path
    else:
        final_path = ''
    return final_path


def is_disabled(item):
    return item.get('disabled', False) is True


def map_to_dict(items):
    return {item.get('key'): item.get('value') \
            for item in items if not is_disabled(item)}


def strip_charset(item):
    item = re.sub(r'charset\=.+', '', str(item))
    return item.replace(';', '').rstrip()


def camelize(string):
    return ''.join(a.capitalize() for a in re.split('([^a-zA-Z0-9])', string) if a.isalnum())


def sanitized(name):
    return name and re.sub('^[^a-zA-Z_]+', '', re.sub('[^0-9a-zA-Z_]', '', name))



# from distutils.core import setup
import setuptools

from peeman.__version__ import __version__


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content


setuptools.setup(
    name='Peeman',
    version=__version__,
    license="MIT",
    zip_safe=False,
    python_requires=">=3.5",
    url='https://github.com/strobes-co/openman',
    keywords=['api', 'openapi', 'rest', 'specification', 'oas', 'documentation'],
    packages=setuptools.find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    install_requires=[
        'apispec==3.1.0',
        'Click==7.0',
        'jsonpath-rw==1.4.0',
        'connexion~=2.6',
        'swagger-ui-bundle==0.0.6',
        'Faker==2.0.4',
        'flask-cors==3.0.8',
    ],
    tests_require=[
        'pytest==5.2.2',
        'coverage==4.5.4',
        'tox==3.14.1',
        'pytest-cov==2.8.1',
    ],
    test_suite='tests',
    description='A Postman to OpenAPI spec converter with mocking facilities',
    long_description=read("README.md"),
    long_description_content_type='text/markdown',
    entry_points={
        'console_scripts': ['peeman=peeman.cli:cli']
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
